==> etlcpp_20.37.2+dfsg.orig.tar.xz: etl-20.37.2/include/etl/combinations.h <==
///\file

/******************************************************************************
The MIT License(MIT)

Embedded Template Library.
https://github.com/ETLCPP/etl
https://www.etlcpp.com

Copyright(c) 2018 John Wellbelove

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files(the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions :

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#ifndef ETL_COMBINATIONS_INCLUDED
#define ETL_COMBINATIONS_INCLUDED

#include "platform.h"
#include "permutations.h"
#include "factorial.h"

///\defgroup combinations combinations
/// combinations<N, K> : Calculates K combinations from N.
///\ingroup maths

namespace etl
{
  //***************************************************************************
  ///\ingroup combinations
  /// Calculates combinations.
  //***************************************************************************
  template <size_t NV, size_t KV>
  struct combinations
  {
    static ETL_CONSTANT size_t value = etl::permutations<NV, KV>::value / etl::factorial<KV>::value;
  };

  template <size_t NV, size_t KV>
  ETL_CONSTANT size_t combinations<NV, KV>::value;

#if ETL_USING_CPP17
  template <size_t NV, size_t KV>
  inline constexpr size_t combinations_v = combinations<NV, KV>::value;
#endif
}

#endif

==> etlcpp_20.44.2+dfsg.orig.tar.xz: etl-20.44.2/include/etl/combinations.h <==
///\file

/******************************************************************************
The MIT License(MIT)

Embedded Template Library.
https://github.com/ETLCPP/etl
https://www.etlcpp.com

Copyright(c) 2018 John Wellbelove

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files(the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and / or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions :

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
******************************************************************************/

#ifndef ETL_COMBINATIONS_INCLUDED
#define ETL_COMBINATIONS_INCLUDED

#include "platform.h"
#include "permutations.h"
#include "factorial.h"

///\defgroup combinations combinations
/// combinations<N, K> : Calculates K combinations from N.
///\ingroup maths

namespace etl
{
  //***************************************************************************
  ///\ingroup combinations
  /// Calculates combinations.
  //***************************************************************************
  template <size_t Value, size_t KV>
  struct combinations
  {
    static ETL_CONSTANT size_t value = etl::permutations<Value, KV>::value / etl::factorial<KV>::value;
  };

  template <size_t Value, size_t KV>
  ETL_CONSTANT size_t combinations<Value, KV>::value;

#if ETL_USING_CPP17
  template <size_t Value, size_t KV>
  inline constexpr size_t combinations_v = combinations<Value, KV>::value;
#endif
}

#endif
